#!/bin/bash

# Copyright (C) since 2014 NTT DATA Corporation
#
# This program is free software; you can redistribute it and/or
# Modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation, version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details

# bash configuration
SCRIPT_DIR=$(cd $(dirname $0);pwd)
PROG=`basename $0`

########################################
#  Local Variable
########################################

# script version
SCRIPT_VERSION="1.0.0"

# script version
TRYCOUNT=3

# sleep time for for-loop
SLEEPTIME=10

# a aws cli wait time (seconds)
WAITTIME=5

# a aws cli max number of messages
MAXNUMBER=10

########################################
# Function
########################################
#
# Message Logging
#
function Logging() {
	[ "x${QUIET}" != "xtrue" ] && echo "$1"
	/usr/bin/logger "${PROG} $1"
}

#
# show usage
#
function usage {
	echo "usage : ${PROG} -p PROFILE [-r REGION] -n QUEUENAME -s SESSIONID [-w WAITTIME] [-m MAXNUMBER] -q [--version] [--help]"
	echo "options:"
	echo "  -p PROFILE      set a aws cli profile name"
	echo "  -r REGION       set a aws cli region"
	echo "  -s SESSIONID    set a job session id"
	echo "  -n QUEUENAME    set a aws cli sqs queue name"
	echo "  -w WAITTIME     set a aws cli wait time (seconds) (defult:${WAITTIME})"
	echo "  -m MAXNUMBER    set a aws cli max number of messages (defult:${MAXNUMBER})"
	echo "  -q              quiet mode"
	echo "  --version       show version"
	echo "  --help          show usage"
}

########################################
# SHELL
########################################

# help chekc
for OPT in $@
do
	case $OPT in
		--version)
			echo ${SCRIPT_VERSION}
			exit 0
			;;
		--help)
			usage
			exit 0
			;;
	esac
done

# check parameters
while getopts "p:r:s:n:w:m:q" OPT
do
	case ${OPT} in
		p)
			PROFILE=$OPTARG
			;;
		r)
			REGION=$OPTARG
			;;
		s)
			SESSIONID=${OPTARG}
			;;
		n)
			QUEUENAME=${OPTARG}
			;;
		w)
			WAITTIME=${OPTARG}
			;;
		m)
			MAXNUMBER=${OPTARG}
			;;
		q)
			QUIET=true
			;;
		*)
			usage
			exit 1
			;;
	esac
done

shift $((OPTIND - 1))

if [ "x${PROFILE}" == "x" ];then
	echo "option (-p) is not specified."
	usage
	exit 1
fi
Logging "target profile ${PROFILE}"

if [ "x${REGION}" != "x" ];then
	ARG_REGION="--r ${REGION}"
fi

if [ "x${QUEUENAME}" == "x" ];then
	echo "option (-n) is not specified."
	usage
	exit 1
fi
Logging "target queue name ${QUEUENAME}"

if [ ! -n "${SESSIONID}" ]; then
	echo "option (-s) is not specified."
	usage
	exit 1
fi
Logging "target session id ${SESSIONID}"

# queue url get
QUEUEURL=`aws --profile ${PROFILE} ${ARG_REGION} sqs get-queue-url --queue-name ${QUEUENAME} | jq -r '.QueueUrl'` 
if [ ! -n "${QUEUEURL}" ]; then
	echo "Failed to get the URL for the queue name ${QUEUENAME}"
 	exit 1
fi
Logging "QUEUEURL : ${QUEUEURL}"
Logging "WAITTIME : ${WAITTIME}"
Logging "MAXNUMBER : ${MAXNUMBER}"

# message receive
MESSAGE_FILE="/tmp/aws_sqs_${SESSIONID}.txt"
Logging "Receive Message start"
Logging "output file : ${MESSAGE_FILE}"

RETURNCODE=1

for i in `seq 1 ${TRYCOUNT}`
do
	Logging "Receive Message $i time"
	aws --profile ${PROFILE} ${ARG_REGION} sqs receive-message --wait-time-seconds ${WAITTIME} --queue-url ${QUEUEURL} --max-number-of-messages ${MAXNUMBER} | jq -r '.Messages[]' > ${MESSAGE_FILE}
	COUNT=`cat ${MESSAGE_FILE} | grep ReceiptHandle | wc -l`
	if [ ${COUNT} -eq 1 ]; then
		aws --profile ${PROFILE} ${ARG_REGION} sqs delete-message --queue-url ${QUEUEURL} --receipt-handle `cat ${MESSAGE_FILE} | jq -r '.ReceiptHandle'`
		Logging "SQS message receive success"
		RETURNCODE=0
		break
		sleep ${SLEEPTIME}
	else
		Logging "SQS message receive fail"
	fi
done
Logging "Receive Message end"

########################################
# Termination Processing
########################################
exit ${RETURNCODE}

